#include "stc.h"
#include "usb.h"
#include "uart.h"
#include "timer.h"
#include "clock.h"

void sys_init();
void scan_key();
void check_init();
void check_isp();

BOOL g_fKeyStable;
BOOL g_fKeyState;
BYTE g_bKeyDebounce;
BYTE g_bOPTION_0;
BYTE g_bOPTION_1;

BYTE code OPTION[16] = 
{
    0x5a, 0xa5,
    0xc3,           //(1)bit0(P3.2 power ctrl)              0:disable           1:enable
                    //(1)bit1(P3.3 LED ctrl)                0:disable           1:enable
                    //(0)bit2(P3.6 RxD_2/S-TxD mode)        0:input-only        1:quasi-bidir
                    //(0)bit3(P1.0 RxD2/S-TxD2 mode)        0:input-only        1:quasi-bidir
                    //(0)bit4(P3.7 TxD_2/S-RxD idle mode)   0:open_drain        1:push-pull
                    //(0)bit5(P1.1 TxD2/S-RxD2 idle mode)   0:open_drain        1:push-pull
                    //(1)bit6(P3.7 TxD_2/S-RxD act mode)    0:open_drain        1:push-pull
                    //(1)bit7(P1.1 TxD2/S-RxD2 act mode)    0:open_drain        1:push-pull
    0x02,           //(0)bit0(P1.0/P1.1 switch ctrl)        0:maintain          1:switch
                    //(1)bit1(uart select)                  0:UART1(P3.6/P3.7)  1:UART2(P1.0/P1.1)
    0,0,0,0,0,0,0,0,0,0,0,0,
};

#define KEY_PRESSED()           (g_fKeyStable && !g_fKeyState)
#define KEY_RELEASEED()         (g_fKeyStable && g_fKeyState)

void main()
{
    sys_init();
    clk_init();
    timer_init();
    uart_init();
    usb_init();
    
#ifdef ISPPWRCTRL
    check_init();
#endif

    EA = 1;
    
    while (1)
    {
        if (DeviceState != DEVSTATE_CONFIGURED)
            continue;
            
        uart_polling();
        
        if (f1ms)
        {
            f1ms = 0;
            scan_key();
        }
        
        if (f10ms)
        {
            f10ms = 0;
            
#ifdef ISPPWRCTRL
            check_isp();
#endif
        }
    }
}

void sys_init()
{
    if ((OPTION[0] == 0x5a) &&
        (OPTION[1] == 0xa5))
    {
        g_bOPTION_0 = OPTION[2];
        g_bOPTION_1 = OPTION[3];
    }
    else
    {
        g_bOPTION_0 = 0xc3;
        g_bOPTION_1 = 0x02;
    }
    
    P_SW2 = 0x80;
    P_SW1 = 0x00;
    AUXR = 0x00;
    
    P0M1 = 0x00;   P0M0 = 0x00;
    P1M1 = 0x00;   P1M0 = 0x00;
    P2M1 = 0x00;   P2M0 = 0x00;
    P3M1 = 0x00;   P3M0 = 0x00;
    P4M1 = 0x00;   P4M0 = 0x00;
    P5M1 = 0x00;   P5M0 = 0x00;
    P6M1 = 0x00;   P6M0 = 0x00;
    P7M1 = 0x00;   P7M0 = 0x00;
    
    P1DR &= ~0x02;              //ʹP1.1
    P3DR &= ~0x80;              //ʹP3.7

    if (g_bOPTION_1 & 0x01)
        P_SWX1 |= 0x02;
    else
        P_SWX1 &= ~0x02;

    PLED_MODE();
    PSTXD_MODE();
    PSTXD2_MODE();
    PSRXD_IDLEMODE();
    PSRXD2_IDLEMODE();
    
    DOWNLOAD = 1;
    LED_POWER = 1;              //LED
    
    g_fKeyState = !DOWNLOAD;    //ǿưɨ
    g_fKeyStable = 0;
    g_bKeyDebounce = 30;
}

#ifdef ISPPWRCTRL
    #define ISPSTG_IDLE         0
    #define ISPSTG_CNT7F        1
    #define ISPSTG_PWROFF       2

    #define CNT7F_MAX           16          //⵽7Fĸ
    #define TIMEOUT_DETECT      2000        //7Fĳʱʱ(2000ms)
    #define TIME_PWROFF         500         //Զͣʱ(500ms)
    
    BOOL IspUartSet;
    BYTE IspStage;
    BYTE IspWaitX10ms;
    BYTE IspCount7F;
    
    void check_init()
    {
        IspUartSet = 0;
        IspStage = ISPSTG_IDLE;
        IspWaitX10ms = 0;
        IspCount7F = 0;
        
        SVCC_ON();
    }
    
    /*ÿ10msһ*/
    void check_isp()
    {
        switch (IspStage)
        {
        default:
        case ISPSTG_IDLE:
            if (KEY_RELEASEED())
            {
                SVCC_ON();
            }
            if (IspUartSet)
            {
                IspUartSet = 0;
                IspWaitX10ms = 0;
                IspCount7F = 0;
                IspStage = ISPSTG_CNT7F;
            }
            break;
        case ISPSTG_CNT7F:
            if (IspWaitX10ms >= (TIMEOUT_DETECT/10))
            {
                IspStage = ISPSTG_IDLE;
            }
            else
            {
                IspWaitX10ms++;
                if (IspCount7F >= CNT7F_MAX)
                {
                    SVCC_OFF();
                    IspUartSet = 0;
                    IspWaitX10ms = 0;
                    IspStage = ISPSTG_PWROFF;
                }
            }
            break;
        case ISPSTG_PWROFF:
            if (IspWaitX10ms >= (TIME_PWROFF/10))
            {
                SVCC_ON();
                IspStage = ISPSTG_IDLE;
            }
            else
            {
                IspWaitX10ms++;
            }
            break;
        }
    }
#endif

void scan_key()
{
    if (!(g_bOPTION_0 & 0x01))
        return;

    if (DOWNLOAD != g_fKeyState)
    {
        g_fKeyState = DOWNLOAD;
        g_fKeyStable = 0;
        g_bKeyDebounce = 30;
    }
    else
    {
        if (g_bKeyDebounce)
        {
            if (--g_bKeyDebounce == 0)
            {
                g_fKeyStable = 1;
                
                if (KEY_PRESSED())
                {
                    SVCC_OFF();
                }
                
                if (KEY_RELEASEED())
                {
                    SVCC_ON();
                }
            }
        }
    }
}
