#include "stc.h"
#include "uart.h"
#include "usb.h"
#include "usb_desc.h"
#include "util.h"

BOOL UartBusy;

void uart_init()
{
    LineCoding.dwDTERate = REV4(115200);
    LineCoding.bCharFormat = 0;
    LineCoding.bParityType = 0;
    LineCoding.bDataBits = 8;
    LineCoding.bLineState = 0;
    
    if (UART1_SEL())
    {
        P_SW1 = (P_SW1 & ~0xc0) | 0x40;
        SCON = 0x50;
        AUXR |= S1ST2;
        ES = 1;
    }
    else
    {
        P_SW2 &= ~0x01;
        S2CON = 0x50;
        IE2 |= ES2;
    }
    AUXR |= T2x12;
    AUXR |= T2R;

//    T2L = BR(115200);
//    T2H = BR(115200) >> 8;
    uart_set_baud();
}

void uart1_isr() interrupt UART1_VECTOR
{
    if (TI)
    {
        TI = 0;
        PSRXD_IDLEMODE();
        
        if (UART1_SEL())
            UartBusy = 0;
    }

    if (RI)
    {
        RI = 0;

        if (UART1_SEL())
            TxBuffer[TxWptr++] = SBUF;
    }
}

void uart2_isr() interrupt UART2_VECTOR
{
    if (S2CON & S2TI)
    {
        S2CON &= ~S2TI;
        PSRXD2_IDLEMODE();

        if (UART2_SEL())
            UartBusy = 0;
    }

    if (S2CON & S2RI)
    {
        S2CON &= ~S2RI;

        if (UART2_SEL())
            TxBuffer[TxWptr++] = S2BUF;
    }
}

void uart_set_parity(BYTE parity)
{
    switch (parity)
    {
    case NONE_PARITY:
        if (UART1_SEL())
        {
            SM0 = 0;
        }
        else
        {
            S2CON &= ~S2SM0;
        }
        break;
    case ODD_PARITY:
    case EVEN_PARITY:
    case MARK_PARITY:
        if (UART1_SEL())
        {
            SM0 = 1;
            TB8 = 1;
        }
        else
        {
            S2CON |= S2SM0;
            S2CON |= S2TB8;
        }
        break;
    case SPACE_PARITY:
        if (UART1_SEL())
        {
            SM0 = 1;
            TB8 = 0;
        }
        else
        {
            S2CON |= S2SM0;
            S2CON &= ~S2TB8;
        }
        break;
    }
}

void uart_set_baud()
{
    DWORD baud;
    WORD temp;

    baud = reverse4(LineCoding.dwDTERate);
    temp = BR(baud);
    T2L = temp;
    T2H = temp >> 8;
}

void uart_polling()
{
    BYTE dat;
    BYTE cnt;
    BYTE xdata buf[EP1IN_SIZE];

    if (DeviceState != DEVSTATE_CONFIGURED)
        return;

    if (!UsbInBusy && (TxRptr != TxWptr))
    {
        cnt = 0;
        while (TxRptr != TxWptr)
        {
            buf[cnt++ + 1] = TxBuffer[TxRptr++];
            if (cnt == EP1IN_SIZE - 1) break;
        }
        buf[0] = cnt;
        while (cnt != EP1IN_SIZE - 1)
            buf[cnt++ + 1] = 0;

        IE2 &= ~EUSB;
        UsbInBusy = 1;
        usb_write_reg(INDEX, 1);
        cnt = 0;
        while (cnt != EP1IN_SIZE)
            usb_write_reg(FIFO1, buf[cnt++]);
        usb_write_reg(INCSR1, INIPRDY);
        IE2 |= EUSB;
    }

    if (!UartBusy && (RxRptr != RxWptr))
    {
        if ((RxRptr % EP1IN_SIZE) == 0)
        {
            if (strcmp(&RxBuffer[RxRptr], USER_STCISPCMD) == 0)
            {
                USBCON = 0x00;
                USBCLK = 0x00;
                IRC48MCR = 0x00;

                sleep_ms(10);

                IAP_CONTR = 0x60;
                while (1);
            }

            RxSize = RxBuffer[RxRptr++];
        }
        else
        {
            dat = RxBuffer[RxRptr++];
            if (RxSize)
            {
                BOOL tb8;
                
                RxSize--;

                UartBusy = 1;
                switch (LineCoding.bParityType)
                {
                case NONE_PARITY:
                case SPACE_PARITY:
                    tb8 = 0;
                    break;
                case ODD_PARITY:
                    ACC = dat;
                    tb8 = !P;
                    break;
                case EVEN_PARITY:
                    ACC = dat;
                    tb8 = P;
                    break;
                case MARK_PARITY:
                    tb8 = 1;
                    S2CON |= S2TB8;
                    break;
                }
                if (UART1_SEL())
                {
                    TB8 = tb8;
                    PSRXD_ACTMODE();
                    SBUF = dat;
                }
                else
                {
                    if (tb8)
                        S2CON |= S2TB8;
                    else
                        S2CON &= ~S2TB8;
                    PSRXD2_ACTMODE();
                    S2BUF = dat;
                }

//                while (UartBusy);
        
#ifdef ISPPWRCTRL
                //ISPʱ7Fм
                {
                    extern BYTE IspCount7F;
                    
                    if (dat == 0x7f)
                        IspCount7F++;
                    else
                        IspCount7F = 0;
                }
#endif
            }
        }
    }

    if (UsbOutBusy)
    {
        IE2 &= ~EUSB;
        if (RxWptr - RxRptr < 256 - EP1OUT_SIZE)
        {
            UsbOutBusy = 0;
            usb_write_reg(INDEX, 1);
            usb_write_reg(OUTCSR1, 0);
        }
        IE2 |= EUSB;
    }
}
